@@ -18,10 +18,15 @@ without_internet <- function (expr) {
 
 stopRequest <- function (req, handle, refresh) {
     out <- paste(req$method, req$url)
-    body <- req$options$postfields
-    if (!is.null(body)) {
+    body <- requestBody(req)
+    if (length(body) > 0) {
         out <- paste(out, rawToChar(body))
     }
+    if (!is.null(req$mockfile)) {
+        ## Poked in here by mockRequest for ease of debugging
+        ## Append it to the end.
+        out <- paste0(out, " (", req$mockfile, ")")
+    }
     stop(out, call.=FALSE)
 }
 
